"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ssl_pinning_common_1 = require("./ssl-pinning.common");
var types_1 = require("tns-core-modules/utils/types");
var peer = {
    enabled: false,
    allowInvalidCertificates: false,
    validatesDomainName: true
};
var SslPinning = (function (_super) {
    __extends(SslPinning, _super);
    function SslPinning() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SslPinning.enableSSLPinning = function (options) {
        if (peer.host && peer.certificate) {
            return;
        }
        var certificate;
        var inputStream;
        try {
            var file = new java.io.File(options.certificate);
            inputStream = new java.io.FileInputStream(file);
            var x509Certificate = java.security.cert.CertificateFactory.getInstance('X509').generateCertificate(inputStream);
            peer.x509Certificate = x509Certificate;
            certificate = okhttp3.CertificatePinner.pin(x509Certificate);
            inputStream.close();
        }
        catch (error) {
            console.error('nativescript-ssl-pinning > enableSSLPinning error', error);
            return;
        }
        peer.host = options.host;
        peer.certificate = certificate;
        peer.allowInvalidCertificates = options.allowInvalidCertificates;
        peer.validatesDomainName = options.validatesDomainName;
        peer.enabled = true;
        this.initializeClient();
        console.log('nativescript-ssl-pinning > Enabled SSL pinning');
    };
    SslPinning.disableSSLPinning = function () {
        peer.enabled = false;
        this.initializeClient();
        console.log('nativescript-ssl-pinning > Disabled SSL pinning');
    };
    SslPinning.getClient = function () {
        if (!this.Client) {
            this.initializeClient();
        }
        return this.Client;
    };
    SslPinning.initializeClient = function () {
        var client = new okhttp3.OkHttpClient.Builder();
        if (peer.enabled && (peer.host || peer.certificate)) {
            var spec = okhttp3.ConnectionSpec.MODERN_TLS;
            client.connectionSpecs(java.util.Collections.singletonList(spec));
            var pinner = new okhttp3.CertificatePinner.Builder();
            pinner.add(peer.host, [peer.certificate]);
            client.certificatePinner(pinner.build());
            if (!peer.allowInvalidCertificates) {
                try {
                    var x509Certificate = peer.x509Certificate;
                    var keyStore = java.security.KeyStore.getInstance(java.security.KeyStore.getDefaultType());
                    keyStore.load(null, null);
                    keyStore.setCertificateEntry('CA', x509Certificate);
                    var keyManagerFactory = javax.net.ssl.KeyManagerFactory.getInstance('X509');
                    keyManagerFactory.init(keyStore, null);
                    var keyManagers = keyManagerFactory.getKeyManagers();
                    var trustManagerFactory = javax.net.ssl.TrustManagerFactory.getInstance(javax.net.ssl.TrustManagerFactory.getDefaultAlgorithm());
                    trustManagerFactory.init(keyStore);
                    var sslContext = javax.net.ssl.SSLContext.getInstance('TLS');
                    sslContext.init(keyManagers, trustManagerFactory.getTrustManagers(), new java.security.SecureRandom());
                    client.sslSocketFactory(sslContext.getSocketFactory());
                }
                catch (error) {
                    console.error('nativescript-ssl-pinning > client.allowInvalidCertificates error', error);
                }
            }
            try {
                client.hostnameVerifier(new javax.net.ssl.HostnameVerifier({
                    verify: function (hostname, session) {
                        var pp = session.getPeerPrincipal().getName();
                        var hv = javax.net.ssl.HttpsURLConnection.getDefaultHostnameVerifier();
                        var wildCardDomains = peer.host.split('.').splice(-2).join('.') === hostname.split('.').splice(-2).join('.');
                        return (wildCardDomains ||
                            hv.verify(peer.host, session) &&
                                peer.host === hostname &&
                                peer.host === session.getPeerHost() &&
                                pp.indexOf(peer.host) !== -1);
                    },
                }));
            }
            catch (error) {
                console.error('nativescript-ssl-pinning > client.validatesDomainName error', error);
            }
        }
        this.Client = client.build();
    };
    SslPinning.request = function (opts) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                if (!(opts.headers && opts.headers['Content-Type'])) {
                    opts.headers = {};
                }
                opts.headers['Content-Type'] = opts.headers['Content-Type'] || 'application/json';
                var client = _this.getClient();
                var requestBuilder_1 = new okhttp3.Request.Builder();
                requestBuilder_1.url(opts.url);
                Object.keys(opts.headers).forEach(function (key) { return requestBuilder_1.addHeader(key, opts.headers[key]); });
                if ((['GET', 'HEAD'].indexOf(opts.method) !== -1) || (opts.method === 'DELETE' && !types_1.isDefined(opts.body))) {
                    requestBuilder_1[ssl_pinning_common_1.METHODS[opts.method]]();
                }
                else {
                    requestBuilder_1[ssl_pinning_common_1.METHODS[opts.method]](_this.getRequestBody(opts));
                }
                if (opts.allowLargeResponse) {
                    android.os.StrictMode.setThreadPolicy(android.os.StrictMode.ThreadPolicy.LAX);
                }
                client.newCall(requestBuilder_1.build()).enqueue(new okhttp3.Callback({
                    onResponse: function (task, response) {
                        var content;
                        var headers = _this.getResponseHeaders(response.headers());
                        var statusCode = response.code();
                        if (response.isSuccessful()) {
                            try {
                                content = JSON.parse(response.body().string());
                            }
                            catch (e) {
                            }
                            resolve({ content: content, statusCode: statusCode, headers: headers });
                        }
                        reject({ status: statusCode, headers: headers, statusText: response.message() });
                    },
                    onFailure: function (task, error) {
                        reject(error);
                    },
                }));
            }
            catch (error) {
                reject(error);
            }
        });
    };
    SslPinning.getRequestBody = function (requestOptions) {
        if (this.isMultipartFormRequest(requestOptions.headers)) {
            console.log('Building Multipart req');
            var multipartBodyBuilder_1 = new okhttp3.MultipartBody.Builder()
                .setType(okhttp3.MultipartBody.FORM);
            (requestOptions.body.toString()).split('&')
                .forEach(function (keyValue) {
                var _a = keyValue.split('='), key = _a[0], value = _a[1];
                multipartBodyBuilder_1.addFormDataPart(key, decodeURIComponent(value));
            });
            return multipartBodyBuilder_1.build();
        }
        return okhttp3.RequestBody.create(okhttp3.MediaType.parse(requestOptions.headers['Content-Type']), JSON.stringify(ssl_pinning_common_1.dataObject(requestOptions.body)));
    };
    SslPinning.getResponseHeaders = function (headers) {
        var mHeaders = {};
        var i, len = headers.size();
        for (i = 0; i < len; i++) {
            var key = headers.name(i);
            headers[key] = headers.value(i);
        }
        return mHeaders;
    };
    return SslPinning;
}(ssl_pinning_common_1.SslPinningCommon));
exports.SslPinning = SslPinning;
//# sourceMappingURL=ssl-pinning.android.js.map